﻿using System.Collections.Generic;
using CognitiveGeology.Components.CognitiveControls.Graph;
using CognitiveGeology.Components.CognitiveControls.Graph.Services;
using CognitiveGeology.Components.CognitiveControls.Model;
using NUnit.Framework;

namespace CognitiveGeology.CognitiveControls.Test.Graph.Services
{
    [TestFixture]
    public class DataRangeAnalyserTests
    {
        private readonly DataRangeAnalyser underTest;
        private DataPointCollection[] testCaseData;
        private RangeAnalysisResult[] expectedResultsForMeasureAxis;
        private RangeAnalysisResult[] expectedResultsForPropertyAxis;

        public DataRangeAnalyserTests()
        {
            this.underTest = new DataRangeAnalyser();
            SetUpTestCaseDataAndExpectedResults();
        }

        [TestCase(0)]
        [TestCase(1)]
        [TestCase(2)]
        [TestCase(3)]
        [TestCase(4)]
        [TestCase(5)]
        [TestCase(6)]
        public void Analyse_TestCase_ProducesExpectedResults_ForMeasureAxis(int index)
        {
            var expected = expectedResultsForMeasureAxis[index];
            var testCase = testCaseData[index];
            
            var actual = underTest.Analyse(testCase, Axis.MeasureAxis);
            
            Assert.AreEqual(expected.Minimum, actual.Minimum);
            Assert.AreEqual(expected.Maximum, actual.Maximum);
            Assert.AreEqual(expected.StepSize, actual.StepSize);
        }
        
        [TestCase(0)]
        [TestCase(1)]
        [TestCase(2)]
        [TestCase(3)]
        [TestCase(4)]
        [TestCase(5)]
        [TestCase(6)]
        public void Analyse_TestCase_ProducesExpectedResults_ForPropertyAxis(int index)
        {
            var expected = expectedResultsForPropertyAxis[index];
            var testCase = testCaseData[index];
            
            var actual = underTest.Analyse(testCase, Axis.PropertyAxis);
            
            Assert.AreEqual(expected.Minimum, actual.Minimum);
            Assert.AreEqual(expected.Maximum, actual.Maximum);
            Assert.AreEqual(expected.StepSize, actual.StepSize);
        }

        private void SetUpTestCaseDataAndExpectedResults()
        {
            var testCaseList = new List<DataPointCollection>();
            var expectedResultListForMeasureAxis = new List<RangeAnalysisResult>();
            var expectedResultListForPropertyAxis = new List<RangeAnalysisResult>();

            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = 2, PropertyValue = 2.5},
                new DataPoint() {MeasureValue = 4, PropertyValue = 5.6},
                new DataPoint() {MeasureValue = 1, PropertyValue = 1.6},
                new DataPoint() {MeasureValue = 5, PropertyValue = 5.8},
                new DataPoint() {MeasureValue = 3, PropertyValue = 5.3},
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = 0, Maximum = 6, StepSize = 1 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult(){Minimum = 0, Maximum = 7, StepSize = 1});

            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = 40, PropertyValue = -2},
                new DataPoint() {MeasureValue = 100, PropertyValue = 2.5},
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = 34, Maximum = 106, StepSize = 3 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult(){Minimum = -3, Maximum = 4, StepSize = 1});

            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = -50, PropertyValue = 16},
                new DataPoint() {MeasureValue = -90, PropertyValue = 25},
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = -94, Maximum = -46, StepSize = 2 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult(){Minimum = 15, Maximum = 26, StepSize = 1});

            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = 40, PropertyValue = 1.6},
                new DataPoint() {MeasureValue = 40, PropertyValue = 1.3},
                new DataPoint() {MeasureValue = 40, PropertyValue = 1.4},
                new DataPoint() {MeasureValue = 40, PropertyValue = 1.8},
                new DataPoint() {MeasureValue = 100, PropertyValue = 2.1},
                new DataPoint() {MeasureValue = 100, PropertyValue = 2.7},
                new DataPoint() {MeasureValue = 100, PropertyValue = 2.3},
                new DataPoint() {MeasureValue = 100, PropertyValue = 2.5},
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = 34, Maximum = 106, StepSize = 3 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult() { Minimum = 0, Maximum = 4, StepSize = 1 });


            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = 16.56, PropertyValue = 150},
                new DataPoint() {MeasureValue = 78.45, PropertyValue = 152},
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = 10, Maximum = 85, StepSize = 3 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult() { Minimum = 149, Maximum = 153, StepSize = 1 });

            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = -24.23, PropertyValue = 3.046 },
                new DataPoint() {MeasureValue = -67.34, PropertyValue = 3.053 },
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = -72, Maximum = -20, StepSize = 2 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult() { Minimum = 2, Maximum = 5, StepSize = 1 });

            testCaseList.Add(new DataPointCollection()
            {
                new DataPoint() {MeasureValue = 25, PropertyValue = 2.0013},
                new DataPoint() {MeasureValue = -75, PropertyValue = 2.0018},
            });

            expectedResultListForMeasureAxis.Add(new RangeAnalysisResult() { Minimum = -85, Maximum = 35, StepSize = 5 });
            expectedResultListForPropertyAxis.Add(new RangeAnalysisResult() { Minimum = 1, Maximum = 4, StepSize = 1 });

            testCaseData = testCaseList.ToArray();
            expectedResultsForMeasureAxis = expectedResultListForMeasureAxis.ToArray();
            expectedResultsForPropertyAxis = expectedResultListForPropertyAxis.ToArray();
        }
    }
}