﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Forms;
using CognitiveGeology.Components.CognitiveControls.Model;
using Orientation = System.Windows.Controls.Orientation;

namespace CognitiveControls.TestHarness
{
    public partial class SplineGraphTestHarness : Form
    {
        private double minimumValue, maximumValue, stepValue;

        private DataSource dataSource;

        public SplineGraphTestHarness()
        {
            InitializeComponent();
            dataSource = new DataSource();
            dataSource.LoadData("Sample", "Data\\data.csv");
            dataSource.LoadData("Real", "Data\\realdata.csv");
            ddMeasure.ValueMember = "Value";
            ddMeasure.DisplayMember = "Text";
            ddProperty.ValueMember = "Value";
            ddProperty.DisplayMember = "Text";
            BindDataSetSelector();
        }

        private void BindDataSetSelector()
        {
            ddDataSet.DataSource = new BindingSource(dataSource.Data, null);
            ddDataSet.DisplayMember = "Key";
            ddDataSet.ValueMember = "Key";
        }
        
        private void SelectDataSet(string key)
        {
            var dataSet = dataSource.Data[key];

            var columnList = dataSet[0];

            ddMeasure.DataSource = new BindingSource(GetListItems(columnList, true), null);
            ddProperty.DataSource = new BindingSource(GetListItems(columnList, false), null);

            BindSelectedData(dataSet);
        }

        private void BindGraph(DataPointCollection data, string primaryAxisLabel, string secondaryAxisLabel)
        { 
            splineGraphControl1.Bind(data);
            splineGraphControl1.PrimaryAxisHeader = primaryAxisLabel;
            splineGraphControl1.SecondaryAxisHeader = secondaryAxisLabel;
            
            txtMinimum.Text = splineGraphControl1.PrimaryAxisMinimum.ToString();
            txtMaximum.Text = splineGraphControl1.PrimaryAxisMaximum.ToString();
            txtStep.Text = splineGraphControl1.MeasureStep.ToString();
        }

        private DataPointCollection CreateDataPointCollection(string[][] dataSet, int measureIndex, int propertyIndex)
        {
            var dataPointCollection = new DataPointCollection();
            for (var i = 1; i < dataSet.Length; i++)
            {
                var row = dataSet[i];

                try
                {
                    var measureValue = Convert.ToDouble(row[measureIndex]);
                    var propertyValue = Convert.ToDouble(row[propertyIndex]);
                    dataPointCollection.Add(new DataPoint() { MeasureValue = measureValue, PropertyValue = propertyValue });
                }
                catch 
                {
                }
            }
            return dataPointCollection;
        }

        private double? ParseNumber(string text)
        {
            double result;
            if (double.TryParse(text, NumberStyles.Any, new NumberFormatInfo(), out result))
            {
                return result;
            }
            return null;
        }

        private IEnumerable<ListItem> GetListItems(string[] items, bool isMeasure)
        {
            var measureValues = new string[] { "X", "Y", "Z", "I", "J", "K" };
            
            for (var i = 0; i < items.Length; i++)
            {
                var value = items[i];
                if (measureValues.Contains(value))
                {
                    if (isMeasure)
                    {
                        yield return GetListItem(value, i);
                    }
                }
                else
                {
                    if (!isMeasure)
                    {
                        yield return GetListItem(value, i);
                    }
                }
            }
        }

        private ListItem GetListItem(string item, int index)
        {
            return new ListItem() { Text = item, Value = index };
        }

        private void BindSelectedData(string[][] dataSet)
        {
            var selectedMeasure = (ListItem) ddMeasure.SelectedItem;
            var selectedProperty = (ListItem) ddProperty.SelectedItem;

            BindGraph(CreateDataPointCollection(dataSet, selectedMeasure.Value, selectedProperty.Value), selectedMeasure.Text, selectedProperty.Text);
            BindMapControl(dataSet);
        }

        private void BindMapControl(string[][] dataSet)
        {
            var points = dataSet.SkipWhile(row => row[0]=="X").Select(row => new Point(Convert.ToDouble(row[0]), Convert.ToDouble(row[1]))).Distinct().ToList();
            mapDirectionControl1.Bind(points, "X", "Y");
        }

        #region UI Events

        private void ddDataSet_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selected = (KeyValuePair<string, string[][]>)ddDataSet.SelectedItem;
            SelectDataSet(selected.Key);
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            var selected = (KeyValuePair<string,string[][]>)ddDataSet.SelectedItem;
            var dataSet = selected.Value;

            BindSelectedData(dataSet);
        }

        private void SplineGraphTestHarness_SizeChanged(object sender, EventArgs e)
        {
            elementHost1.Width = this.Width - 350;
        }

        private void rdVertical_CheckedChanged(object sender, EventArgs e)
        {
            if (rdVertical.Checked)
            {
                splineGraphControl1.GraphOrientation = Orientation.Vertical;
            }
        }

        private void rdHorizontal_CheckedChanged(object sender, EventArgs e)
        {
            if (rdHorizontal.Checked)
            {
                splineGraphControl1.GraphOrientation = Orientation.Horizontal;
            }
            
        }

        private void txtMinimum_TextChanged(object sender, EventArgs e)
        {
            double? number = ParseNumber(txtMinimum.Text);
            if (number.HasValue)
            {
                minimumValue = number.Value;
            }
            else
            {
                txtMinimum.Text = minimumValue.ToString();
            }
            splineGraphControl1.PrimaryAxisMinimum = number;
            
            switch (((ListItem) ddMeasure.SelectedItem).Text)
            {
                case "X":
                    mapDirectionControl1.PrimaryAxisMinimum = number;
                    break;
                case "Y":
                    mapDirectionControl1.SecondaryAxisMinimum = number;
                    break;
            }
        }
        
        private void txtMaximum_TextChanged(object sender, EventArgs e)
        {
            double? number = ParseNumber(txtMaximum.Text);
            if (number.HasValue)
            {
                maximumValue = number.Value;
            }
            else
            {
                txtMaximum.Text = maximumValue.ToString();
            }
            splineGraphControl1.PrimaryAxisMaximum = number;

            switch (((ListItem)ddMeasure.SelectedItem).Text)
            {
                case "X":
                    mapDirectionControl1.PrimaryAxisMaximum = number;
                    break;
                case "Y":
                    mapDirectionControl1.SecondaryAxisMaximum = number;
                    break;
            }
        }

        private void txtStep_TextChanged(object sender, EventArgs e)
        {
            double? number = ParseNumber(txtStep.Text);
            if (number.HasValue)
            {
                stepValue = number.Value;
            }
            else
            {
                txtStep.Text = stepValue.ToString();
            }
            splineGraphControl1.MeasureStep = number;
        }

        #endregion

        private void btnFit_Click(object sender, EventArgs e)
        {
            splineGraphControl1.FitLine();
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            splineGraphControl1.ResetLine();
        }
    }

    struct ListItem
    {
        public string Text { get; set; }
        public int Value { get; set; }
    }
}
