﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Media;
using CognitiveGeology.Components.CognitiveControls.Model;
using CognitiveGeology.Core.DTO;

namespace CognitiveControls.TestHarness
{
    public partial class StratiGraphTestHarness : Form
    {
        private Color[] colorsToChooseFrom;
        private ZoneDto[] zones;
        private DataPoint[] dataPoints;
        
        public StratiGraphTestHarness()
        {
            InitializeComponent();
            var dataSource = new DataSource();
            dataSource.LoadData("Stratigraphy", "Data\\stratigraphy.csv");
            colorsToChooseFrom = new Color[] { Colors.DarkSlateGray, Colors.DarkBlue, Colors.DarkRed, Colors.DarkCyan, Colors.DarkSlateBlue, Colors.DarkGreen, Colors.DarkMagenta };
            
            PrepareData(dataSource.Data["Stratigraphy"]);

            stratiGraphControl1.Bind(zones);
            stratiGraphControl1.Bind(dataPoints);
        }

        private void PrepareData(IEnumerable<string[]> data)
        {
            var colourDictionary = new Dictionary<string, Color>();
            var zoneDictionary = new Dictionary<string, ZoneDto>();
            var subzoneDictionary = new Dictionary<string, ZoneDto>();
            var dataPointList = new List<DataPoint>();
            var subZoneList = new List<ZoneDto>();

            byte colorIndex = 0;
            byte colorShade = 55;

            ZoneDto currentZone = null, currentSubzone = null;
            int k = 0;
            int subZoneCount = 0;
            foreach (var row in data)
            {
                k = Convert.ToInt32(row[0]);

                var zoneName = row[1];
                if (zoneDictionary.ContainsKey(zoneName))
                {
                    currentZone = zoneDictionary[zoneName];
                }
                else
                {
                    if (currentZone != null)
                    {
                        currentZone.MaxK = k - 1;
                        currentZone.Subzones = subZoneList.ToArray();
                        subZoneList.Clear();
                        subZoneCount = 0;
                    }
                    var colour = colorsToChooseFrom[colorIndex];
                    var zoneColor = Color.FromArgb(128, colour.R, colour.G, colour.B);
                    currentZone = new ZoneDto() { Name = zoneName, ColourCode =  zoneColor.ToString(), MinK = k };
                    zoneDictionary.Add(zoneName, currentZone);
                    colourDictionary.Add(zoneName, colour);
                    colorIndex++;
                }

                var subzoneName = row[2];
                if (!subzoneDictionary.ContainsKey(subzoneName))
                {
                    subZoneCount++;
                    if (currentSubzone != null)
                    {
                        currentSubzone.MaxK = k - 1;
                    }
                    var colour = colourDictionary[currentZone.Name];
                    var subzoneColor = Color.FromArgb(colorShade, colour.R, colour.G, colour.B);
                    currentSubzone = new ZoneDto() { Name = subzoneName, ColourCode = subzoneColor.ToString(), MinK = k };
                    subzoneDictionary.Add(subzoneName, currentSubzone);
                    subZoneList.Add(currentSubzone);
                    colorShade += 55;
                }
                
                var value = Convert.ToDouble(row[3]);
                
                dataPointList.Add(new DataPoint(){MeasureValue = k, PropertyValue = value });
            }
            if (currentZone != null)
            {
                currentZone.MaxK = k;

                if (currentSubzone != null)
                {
                    currentSubzone.MaxK = k;
                    currentZone.Subzones = subZoneList.ToArray();
                }
            }

            zones = zoneDictionary.Values.ToArray();
            dataPoints = dataPointList.ToArray();
        }
    }
}
