﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using CognitiveGeology.Core.Model;
using Syncfusion.UI.Xaml.Charts;

namespace CognitiveGeology.Components.CognitiveControls.Graph
{
    /// <summary>
    /// Interaction logic for SplineGraphControl.xaml
    /// </summary>
    public partial class SplineGraphControl : UserControl, INotifyPropertyChanged
    {
        private const int SymbolWidth = 35;
        private const int SymbolHeight = 20;

        private Orientation graphOrientation;

        //here is where the properties for linear and log should go?

        private bool locked;
        private double? primaryAxisMaximum;
        private double? primaryAxisMinimum;
        private double? secondaryAxisMinimum;
        private double? secondaryAxisMaximum;
        private double pointProximityTolerance;
        private string primaryAxisHeader;
        private string secondaryAxisHeader;

        public event PropertyChangedEventHandler PropertyChanged;
        public event TrendChangedEventHandler TrendChanged;

        public SplineGraphControl()
        {
            locked = true;
            DataPoints = new PropertyValuePointCollection();
            BinnedPoints = new PropertyValuePointCollection();
            MidPoints = new PropertyValuePointCollection();
            TrendDataPoints = new UserDefinedPointCollection();
            LineDataPoints = new PropertyValuePointCollection();
            SelectedDataPoints = new PropertyValuePointCollection();

            Initialize();
        }

        private void Initialize()
        {
            InitializeComponent();
        }

        public double? PrimaryAxisMinimum
        {
            get { return primaryAxisMinimum; }
            set
            {
                primaryAxisMinimum = value;
                NotifyPropertyChanged("PrimaryAxisMinimum");
            }
        }

        public double? PrimaryAxisMaximum
        {
            get { return primaryAxisMaximum; }
            set
            {
                primaryAxisMaximum = value;
                NotifyPropertyChanged("PrimaryAxisMaximum");
            }
        }

        public string PrimaryAxisHeader
        {
            get { return primaryAxisHeader; }
            set
            {
                primaryAxisHeader = value;
                NotifyPropertyChanged("PrimaryAxisHeader");
            }
        }

        public string SecondaryAxisHeader
        {
            get { return secondaryAxisHeader; }
            set
            {
                secondaryAxisHeader = value;
                NotifyPropertyChanged("SecondaryAxisHeader");
            }
        }

        public double? SecondaryAxisMinimum
        {
            get { return secondaryAxisMinimum; }
            set
            {
                secondaryAxisMinimum = value;
                NotifyPropertyChanged("SecondaryAxisMinimum");
            }
        }

        public double? SecondaryAxisMaximum
        {
            get { return secondaryAxisMaximum; }
            set
            {
                secondaryAxisMaximum = value;
                NotifyPropertyChanged("SecondaryAxisMaximum");
            }
        }

        public double? PointProximityTolerance
        {
            get { return pointProximityTolerance; }
            set
            {
                if (value == null) return;
                pointProximityTolerance = value.Value;
                NotifyPropertyChanged("PointProximityTolerance");
            }
        }

        public Orientation GraphOrientation
        {
            get
            {
                return graphOrientation;
            }
            set
            {
                graphOrientation = value;
                NotifyPropertyChanged("Orientation");
            }
        }

        public PropertyValuePointCollection DataPoints { get; private set; }

        public PropertyValuePointCollection BinnedPoints { get; private set; }

        public PropertyValuePointCollection MidPoints { get; private set; }

        public UserDefinedPointCollection TrendDataPoints { get; private set; }

        public PropertyValuePointCollection LineDataPoints { get; private set; }

        public PropertyValuePointCollection SelectedDataPoints { get; private set; }

        public DataPointCollectionSet DataPointCollectionSet
        {
            get { return new DataPointCollectionSet(DataPoints, BinnedPoints, MidPoints, LineDataPoints, TrendDataPoints); }
        }

        public void Unlock()
        {
            locked = false;
        }

        public void Reset(bool fullReset = false)
        {
            locked = true;
            TrendDataPoints.Clear();
            LineDataPoints.Clear();

            if (!fullReset) return;

            DataPoints.Clear();
            MidPoints.Clear();
        }

        private void AddSplinePoint(double x, double y)
        {
            PropertyValuePoint dataPoint = new PropertyValuePoint() { MeasureValue = x, PropertyValue = y };

            TrendDataPoints.AddPoint(dataPoint, pointProximityTolerance);
            if (TrendChanged != null)
            {
                TrendChanged(this, new TrendChangedEventHandlerArgs() { TrendPoints = TrendDataPoints.ToArray() });
            }
        }

        private void SetOrientation()
        {
            if (graphOrientation == Orientation.Horizontal)
            {
                dataSeries.IsTransposed = false;
                pointSeries.IsTransposed = false;
                midPointSeries.IsTransposed = false;
                fittedLineSeries.IsTransposed = false;
                //selectedPointSeries.IsTransposed = false;

                dataSeries.AdornmentsInfo.SymbolWidth = SymbolHeight;
                dataSeries.AdornmentsInfo.SymbolHeight = SymbolWidth;
                var gradientBrush = dataSeries.AdornmentsInfo.SymbolInterior as LinearGradientBrush;
                gradientBrush.EndPoint = new Point(0, 1);
                midPointSeries.AdornmentsInfo.SymbolWidth = 10;
                midPointSeries.AdornmentsInfo.SymbolHeight = 1;
                midPointSeries.AdornmentsInfo.Symbol = ChartSymbol.HorizontalLine;
            }
            else
            {
                dataSeries.IsTransposed = true;
                pointSeries.IsTransposed = true;
                midPointSeries.IsTransposed = true;
                fittedLineSeries.IsTransposed = true;
                //selectedPointSeries.IsTransposed = true;

                dataSeries.AdornmentsInfo.SymbolWidth = SymbolWidth;
                dataSeries.AdornmentsInfo.SymbolHeight = SymbolHeight;
                var gradientBrush = dataSeries.AdornmentsInfo.SymbolInterior as LinearGradientBrush;
                gradientBrush.EndPoint = new Point(1, 0);
                midPointSeries.AdornmentsInfo.SymbolWidth = 1;
                midPointSeries.AdornmentsInfo.SymbolHeight = 10;
                midPointSeries.AdornmentsInfo.Symbol = ChartSymbol.VerticalLine;
            }
        }

        private PropertyValuePoint pointToMove;
        private int pointToMoveIndex;
        private double tolerance = 1;

        private void DataSeries_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (locked)
            {
                return;
            }
            //Register position of the click
            Point positionClick = e.GetPosition(DataGraph);
            positionClick.X -= DataGraph.SeriesClipRect.Left;
            positionClick.Y -= (DataGraph.SeriesClipRect.Top);
            double x = this.DataGraph.PointToValue(this.DataGraph.PrimaryAxis, positionClick);
            double y = this.DataGraph.PointToValue(this.DataGraph.SecondaryAxis, positionClick);
            PropertyValuePoint pt = new PropertyValuePoint(x, y);

            //Find closest point to click
            PropertyValuePoint closest = new PropertyValuePoint(Double.NegativeInfinity, Double.PositiveInfinity);
            double distanceClosest = Double.MaxValue;
            //distance function should probably be moved in the point class later on
            Func<PropertyValuePoint, PropertyValuePoint, double> distance = (p1, p2) => Math.Sqrt(Math.Pow(p1.MeasureValue - p2.MeasureValue, 2) + Math.Pow(p1.PropertyValue - p2.PropertyValue, 2));
            for (int i = 0; i < TrendDataPoints.Count; i++)
            {
                PropertyValuePoint point = TrendDataPoints[i];
                double distanceCurrent = distance(point, pt);
                if (distanceCurrent <= distanceClosest)
                {
                    closest = point;
                    distanceClosest = distanceCurrent;
                    pointToMoveIndex = i;
                }
            }

            //Find out whether it's within the tolerance radius
            if (distance(closest, pt) > tolerance)
            {
                //if it's not, there is no need to remove any datapoint, and the mouseleftup event will take care of the new datapoint
                return;
            }
            //if it is, register the position of that datapoint in memory
            pointToMove = closest;

            SelectedDataPoints.Add(closest);
            changeColorOfSpline(Colors.LightGray);
        }

        private void changeColorOfSpline(Color newColor)
        {
            SolidColorBrush brush = new SolidColorBrush(newColor);
            fittedLineSeries.Interior = brush;
        }

        private void DataGraph_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (locked)
            {
                return;
            }
            for (int i = 0; i < SelectedDataPoints.Count; i++) SelectedDataPoints.RemoveAt(i);
            Point positionClick = e.GetPosition(DataGraph);
            positionClick.X -= DataGraph.SeriesClipRect.Left;
            positionClick.Y -= (DataGraph.SeriesClipRect.Top);
            double x = this.DataGraph.PointToValue(this.DataGraph.PrimaryAxis, positionClick);
            double y = this.DataGraph.PointToValue(this.DataGraph.SecondaryAxis, positionClick);
            PropertyValuePoint currentPosition = new PropertyValuePoint(x, y);

            TrendDataPoints.Remove(pointToMove);

            TrendDataPoints.AddPoint(currentPosition, 0);
            if (TrendChanged != null)
            {
                TrendChanged(this, new TrendChangedEventHandlerArgs() { TrendPoints = TrendDataPoints.ToArray() });
            }

            changeColorOfSpline(Colors.Black);
        }

        private void DataGraph_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (locked || TrendDataPoints.Count <= 2)
            {
                return;
            }
            //Register position of the click
            Point positionClick = e.GetPosition(DataGraph);
            positionClick.X -= DataGraph.SeriesClipRect.Left;
            positionClick.Y -= (DataGraph.SeriesClipRect.Top);
            double x = this.DataGraph.PointToValue(this.DataGraph.PrimaryAxis, positionClick);
            double y = this.DataGraph.PointToValue(this.DataGraph.SecondaryAxis, positionClick);
            PropertyValuePoint pt = new PropertyValuePoint(x, y);

            //Find closest point to click
            PropertyValuePoint closest = new PropertyValuePoint(Double.NegativeInfinity, Double.PositiveInfinity);
            double distanceClosest = Double.MaxValue;
            //distance function should probably be put in the point class later on
            Func<PropertyValuePoint, PropertyValuePoint, double> distance = (p1, p2) => Math.Sqrt(Math.Pow(p1.MeasureValue - p2.MeasureValue, 2) + Math.Pow(p1.PropertyValue - p2.PropertyValue, 2));
            TrendDataPoints.ToList().ForEach(point =>
            {
                double distanceCurrent = distance(point, pt);
                if (distanceCurrent <= distanceClosest)
                {
                    closest = point;
                    distanceClosest = distanceCurrent;
                }
            });

            //Find out whether it's within the tolerance radius
            if (distance(closest, pt) > tolerance)
            {
                //if it's not, remove the last point inserted
                RemoveLastPointInserted();
                return;
            }
            //if it is, remove the datapoint and signal the change
            TrendDataPoints.Remove(closest);
            if (TrendChanged != null)
            {
                TrendChanged(this, new TrendChangedEventHandlerArgs() { TrendPoints = TrendDataPoints.ToArray() });
            }
        }

        private double GradientUpperLimit = 1;
        private double GradientLowerLimit = -1;

        private void ColorBadGradientWarnings()
        {
            ClearBadGradientWarnings();
            for (int i = 0; i < LineDataPoints.Count - 1; i++)
            {
                double gradientNumerator = LineDataPoints[i + 1].PropertyValue - LineDataPoints[i].PropertyValue;
                double gradientDenominator = LineDataPoints[i + 1].MeasureValue - LineDataPoints[i].MeasureValue;
                if (gradientDenominator == 0) return;
                double gradient = gradientNumerator / gradientDenominator;
                if (gradient > GradientUpperLimit)
                {
                    ChartStripLine stripLine = new ChartStripLine()
                    {
                        Start = LineDataPoints[i].MeasureValue,
                        Width = (LineDataPoints[i + 1].MeasureValue - LineDataPoints[i].MeasureValue) * 1.01,
                        Background = new SolidColorBrush(Colors.LightSalmon)
                    };
                    DataGraph.PrimaryAxis.StripLines.Add(stripLine);
                }
                if (gradient < GradientLowerLimit)
                {
                    ChartStripLine stripLine = new ChartStripLine()
                    {
                        Start = LineDataPoints[i].MeasureValue,
                        Width = (LineDataPoints[i + 1].MeasureValue - LineDataPoints[i].MeasureValue) * 1.01,
                        Background = new SolidColorBrush(Colors.LightBlue)
                    };
                    DataGraph.PrimaryAxis.StripLines.Add(stripLine);
                }
            }
        }

        private void ClearBadGradientWarnings()
        {
            for (int i = 0; i < DataGraph.PrimaryAxis.StripLines.Count; i++) DataGraph.PrimaryAxis.StripLines.RemoveAt(i);
        }

        private void RemoveLastPointInserted()
        {
            if (TrendDataPoints.Count == 2) return;
            TrendDataPoints.Remove(TrendDataPoints.Last());
            if (TrendChanged != null)
            {
                TrendChanged(this, new TrendChangedEventHandlerArgs() { TrendPoints = TrendDataPoints.ToArray() });
            }
        }


        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
            switch (propertyName)
            {
                case "Orientation":
                    SetOrientation();
                    break;
            }
        }
    }

    public delegate void TrendChangedEventHandler(object sender, TrendChangedEventHandlerArgs args);

    public class TrendChangedEventHandlerArgs
    {
        public PropertyValuePoint[] TrendPoints { get; set; }
    }
}
