﻿using System.Collections.Generic;
using Accord.Statistics.Distributions.Univariate;
using Accord.Statistics.Testing;

namespace CognitiveGeology.Components.CognitiveControls.Histogram.Services
{
    internal interface IGaussianityTester
    {
        GaussianityTestResult ComputeGaussianity(double[] dataset);
    }

    internal class GaussianityTester : IGaussianityTester
    {
        public GaussianityTestResult ComputeGaussianity(double[] dataset)
        {
            NormalDistribution distribution = NormalDistribution.Standard;
            var kstest = new KolmogorovSmirnovTest(dataset, distribution);
            double statistic = kstest.Statistic;
            double pvalue = kstest.PValue;
            bool significant = kstest.Significant;
            return new GaussianityTestResult(significant, statistic, pvalue);
        }
    }

    internal class GaussianityTestResult
    {
        public GaussianityTestResult(bool significant, double statistic, double pvalue)
        {
            IsSignificant = significant;
            Statistic = statistic;
            PValue = pvalue;
        }

        public bool IsSignificant { get; set; }

        public double Statistic { get; set; }

        public double PValue { get; set; }
    }
    
}
