﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using CognitiveGeology.Core.DTO;
using CognitiveGeology.Core.Model;

namespace CognitiveGeology.Components.CognitiveControls.Stratigraphy
{
    /// <summary>
    /// Interaction logic for StratiGraphControl.xaml
    /// </summary>
    public partial class StratiGraphControl : UserControl
    {
        readonly Dictionary<ZoneDto, SubZoneControl> subzoneDictionary;

        public StratiGraphControl()
        {
            InitializeComponent();
            subzoneDictionary = new Dictionary<ZoneDto, SubZoneControl>();
        }

        public void Bind(ZoneDto[] zones)
        {
            int index = 0;
            foreach (var subzone in zones.SelectMany(zone => zone.Subzones))
            {
                var heightFactor = subzone.MaxK - subzone.MinK + 1;
                var height = new GridLength(heightFactor, GridUnitType.Star);
                var rowDefinition = new RowDefinition(){ Height = height};
                subzonesGrid.RowDefinitions.Add(rowDefinition);

                var color = GetColor(subzone.ColourCode);

                var rectangle = new Rectangle() { Fill = new SolidColorBrush(color), Stroke=new SolidColorBrush(Colors.DimGray), Margin = new Thickness(0,0,4,0) };
                Grid.SetRow(rectangle, index);
                Grid.SetColumn(rectangle, 1);
                subzonesGrid.Children.Add(rectangle);
                
                var subZoneControl = new SubZoneControl();
                Grid.SetRow(subZoneControl, index);
                Grid.SetColumn(subZoneControl, 2);
                subzonesGrid.Children.Add(subZoneControl);
                
                subzoneDictionary.Add(subzone, subZoneControl);

                index++;
            }

            index = 0;
            foreach (var zone in zones)
            {
                var rowspan = zone.Subzones.Length;
                
                var color = GetColor(zone.ColourCode);

                var rectangle = new Rectangle() { Fill = new SolidColorBrush(color), Stroke = new SolidColorBrush(Colors.DimGray), Margin = new Thickness(0, 0, 4, 0) };
                Grid.SetRow(rectangle, index);
                Grid.SetRowSpan(rectangle, rowspan);
                Grid.SetColumn(rectangle, 0);
                subzonesGrid.Children.Add(rectangle);
                index += rowspan;
            }
        }

        public void Bind(IReadOnlyCollection<PropertyValuePoint> dataPoints)
        {
            var minPropertyValue = dataPoints.Min(k => k.PropertyValue);
            var maxPropertyValue = dataPoints.Max(k => k.PropertyValue);

            var buffer = Math.Abs(maxPropertyValue - minPropertyValue)/10;

            foreach (var subzone in subzoneDictionary.Keys)
            {
                var kLayers = dataPoints.Where(k => k.MeasureValue >= subzone.MinK && k.MeasureValue <= subzone.MaxK).ToArray();
                var subZoneControl = subzoneDictionary[subzone];
                subZoneControl.RangeStart = minPropertyValue - buffer;
                subZoneControl.RangeEnd = maxPropertyValue + buffer;
                subZoneControl.Bind(kLayers);
            }
        }

        private static Color GetColor(string colorCode)
        {
            Color color;
            var convertFromString = ColorConverter.ConvertFromString(colorCode);
            if (convertFromString != null)
            {
                color = (Color) convertFromString;
            }
            else
            {
                color = Colors.DarkGray;
            } 
            return color;
        }
    }
}
