﻿using System.Linq;
using System.Windows.Controls;
using CognitiveGeology.Core.Model;
using Syncfusion.UI.Xaml.Charts;

namespace CognitiveGeology.Components.CognitiveControls.Stratigraphy
{
    /// <summary>
    /// Interaction logic for SubZoneControl.xaml
    /// </summary>
    public partial class SubZoneControl : UserControl
    {
        public SubZoneControl()
        {
            InitializeComponent();
        }

        public double RangeStart { get; set; }
        public double RangeEnd { get; set; }

        public void Bind(PropertyValuePoint[] kLayers)
        {
            BindRawValues(kLayers);
            BindMidPoints(kLayers);
        }

        private void BindRawValues(PropertyValuePoint[] kLayers)
        {
            var kMin = kLayers.Select(k => k.MeasureValue).Min();
            var kMax = kLayers.Select(k => k.MeasureValue).Max();

            var xAxis = spectrumGraph.PrimaryAxis as NumericalAxis;
            xAxis.Minimum = RangeStart;
            xAxis.Maximum = RangeEnd;

            var yAxis = spectrumGraph.SecondaryAxis as NumericalAxis;
            yAxis.Minimum = kMin - 1;
            yAxis.Maximum = kMax + 1;

            dataSeries.ItemsSource = kLayers;
        }

        private void BindMidPoints(PropertyValuePoint[] kLayers)
        {
            var kValues = kLayers.Select(k => k.MeasureValue).Distinct();
            var list = (from kValue in kValues let midpoint = kLayers.Select(k => k.PropertyValue).Average() select new PropertyValuePoint() {MeasureValue = kValue, PropertyValue = midpoint}).ToList();
            pointSeries.ItemsSource = list;
        }
    }
}